@@ -5,5 +5,5 @@
 #' @param ... other arguments.
 #' @export
 logLik.sglg <- function(object, ...) {
-    object$llglg
+    return(object$llglg)
 }
